﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.IO;

namespace Ogc.Wfs
{
    namespace Internal
    {
        class Helper
        {
            public const string ToppNs = "http://www.openplans.org/topp";
            public const string OgcNs = "http://www.opengis.net/ogc";
            public const string WfsNs = "http://www.opengis.net/wfs";
            public const string GmlNs = "http://www.opengis.net/gml";

            public static string GetToopPrefix(XmlWriter writer)
            {
                return writer.LookupPrefix(ToppNs);
            }

            public static string GetOgcPrefix(XmlWriter writer)
            {
                return writer.LookupPrefix(OgcNs);
            }

            public static string GetWfsPrefix(XmlWriter writer)
            {
                return writer.LookupPrefix(WfsNs);
            }

            public static string GetGmlPrefix(XmlWriter writer)
            {
                return writer.LookupPrefix(GmlNs);
            }
        }
    }

    public class RequestGetFeatureBoxParameters
    {
        string uri;
        public string Uri
        {
            get { return uri; }
            set { uri = value; }
        }

        List<TypeNameParameter> typeNames = new List<TypeNameParameter>();
        public List<TypeNameParameter> TypeNames
        {
            get { return typeNames; }
        }

        public byte[] ToBytes()
        {
            StringBuilder sb = new StringBuilder();
            XmlWriterSettings s = new XmlWriterSettings();
            s.Indent = true;
            s.OmitXmlDeclaration = true;
            using (XmlWriter writer = XmlWriter.Create(sb, s))
            {
                writer.WriteStartElement("GetFeature");

                writer.WriteAttributeString("service", "WFS");
                writer.WriteAttributeString("version", "1.1.0");

                writer.WriteAttributeString("xmlns","topp",null, Internal.Helper.ToppNs);
                writer.WriteAttributeString("xmlns", "wfs", null, Internal.Helper.WfsNs);
                writer.WriteAttributeString("xmlns", "ogc", null, Internal.Helper.OgcNs);
                writer.WriteAttributeString("xmlns", "gml", null, Internal.Helper.GmlNs);

                foreach (TypeNameParameter i in typeNames)
                {
                    i.Write(writer);
                }
                writer.WriteEndElement();
                writer.Flush();
                return Encoding.UTF8.GetBytes(sb.ToString());
            }
        }
    }

    public class TypeNameParameter
    {
        string typaName;
        public string TypaName
        {
            get { return typaName; }
            set { typaName = value; }
        }

        BBoxFilterParameter filter;
        public BBoxFilterParameter Filter
        {
            get { return filter; }
            set { filter = value; }
        }

        List<string> propertyNames = new List<string>();
        public List<string> PropertyNames
        {
            get { return propertyNames; }
        }
        
        internal void Write(XmlWriter writer)
        {
            writer.WriteStartElement(Internal.Helper.GetWfsPrefix(writer), "Query", Internal.Helper.WfsNs);

            writer.WriteAttributeString("typeName", typaName);

            foreach (string p in propertyNames)
            {
                writer.WriteElementString(Internal.Helper.GetWfsPrefix(writer), "PropertyName", Internal.Helper.WfsNs, p);
            }

            filter.Write(writer);

            writer.WriteEndElement();
        }
    }

    public class BBoxFilterParameter
    {
        string geoPropertyName;
        public string GeoPropertyName
        {
            get { return geoPropertyName; }
            set { geoPropertyName = value; }
        }

        EnvelopeParameter envelope;
        public EnvelopeParameter Envelope
        {
            get { return envelope; }
            set { envelope = value; }
        }

        internal void Write(XmlWriter writer)
        {
            writer.WriteStartElement(Internal.Helper.GetOgcPrefix(writer), "Filter", Internal.Helper.OgcNs);

            WriteBox(writer);

            writer.WriteEndElement();
        }

        private void WriteBox(XmlWriter writer)
        {
            writer.WriteStartElement(Internal.Helper.GetOgcPrefix(writer), "BBOX", Internal.Helper.OgcNs);
            writer.WriteElementString(Internal.Helper.GetOgcPrefix(writer), "PropertyName", Internal.Helper.OgcNs, geoPropertyName);
            envelope.Write(writer);
            writer.WriteEndElement();
        }
    }

    public class EnvelopeParameter
    {
        string srsName;

        public string SrsName
        {
            get { return srsName; }
            set { srsName = value; }
        }
        double minX;

        public double MinX
        {
            get { return minX; }
            set { minX = value; }
        }
        double minY;

        public double MinY
        {
            get { return minY; }
            set { minY = value; }
        }
        double maxX;

        public double MaxX
        {
            get { return maxX; }
            set { maxX = value; }
        }
        double maxY;

        public double MaxY
        {
            get { return maxY; }
            set { maxY = value; }
        }

        internal void Write(XmlWriter writer)
        {
            writer.WriteStartElement(Internal.Helper.GetGmlPrefix(writer), "Envelope", Internal.Helper.OgcNs);

            writer.WriteAttributeString("srsName",srsName);

            writer.WriteElementString(Internal.Helper.GetGmlPrefix(writer), "lowerCorner", Internal.Helper.OgcNs, minX + " " + minY);
            writer.WriteElementString(Internal.Helper.GetGmlPrefix(writer), "upperCorner", Internal.Helper.OgcNs, maxX + " " + maxY);

            writer.WriteEndElement();
        }
    }
}
