﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace Ogc.Wms
{
    public abstract class WmsModel
    {
        protected XmlElement dataNode;

        public XmlElement DataNode
        {
            get { return dataNode; }
            set { dataNode = value; }
        }

        public bool Available
        {
            get
            {
                return dataNode != null;
            }
        }

        public void SetAttr(string name, string value)
        {
            dataNode.SetAttribute(name, value);
        }

        public void SetNodeInnerText(string name, string value)
        {
            XmlElement n = dataNode.SelectSingleNode(name) as XmlElement;
            if (n == null)
                n = AddChildNode(name);
            n.InnerText = value;
        }

        public void AddNodeInnerText(string name, string value)
        {
            AddChildNode(name).InnerText = value;
        }

        private XmlElement AddChildNode(string name)
        {
            XmlElement ret = dataNode.OwnerDocument.CreateElement(name);
            dataNode.AppendChild(ret);
            return ret;
        }

        public T CreateChild<T>(string name) where T : WmsModel
        {
            XmlElement n = dataNode.OwnerDocument.CreateElement(name);
            dataNode.AppendChild(n);
            return Factory.Create<T>(n);
        }

        public T GetChild<T>() where T : WmsModel
        {
            return Factory.Create<T>((XmlElement)dataNode.SelectSingleNode(typeof(T).Name));
        }

        public T CreateChild<T>() where T : WmsModel
        {
            return CreateChild<T>(typeof(T).Name);
        }

        public T[] GetChildren<T>() where T : WmsModel
        {
            Type type = typeof(T);
            return GetChildren<T>(type.Name);
        }

        public T[] GetChildren<T>(string name) where T : WmsModel
        {
            List<T> ret = new List<T>();
            foreach (XmlElement i in dataNode.SelectNodes(name))
            {
                ret.Add(Factory.Create<T>(i));
            }
            return ret.ToArray();
        }

        public string[] GetTextArray(string expr)
        {
            List<string> ret = new List<string>();
            foreach (XmlElement i in dataNode.SelectNodes(expr))
            {
                ret.Add(i.InnerText);
            }
            return ret.ToArray();
        }
    }
}
