﻿using SharpMap.Geometries;
using System.Xml;
using System;
using System.Collections.Generic;

public class GmlGeometryBuilder
{
    XmlNamespaceManager nsmgr;
    XmlDocument doc;

    public GmlGeometryBuilder(XmlDocument doc)
    {
        this.doc = doc;
    }

    public List<SharpMap.Geometries.Geometry> BuildGeometrys(string geoPropertyName)
    {
        this.nsmgr = Helper.BuildNamespaceManager(doc);
        List<SharpMap.Geometries.Geometry> ret = new List<SharpMap.Geometries.Geometry>();
        foreach (XmlNode i in doc.SelectNodes("//" + geoPropertyName, nsmgr))
        {
            ret.Add(BuildGeometry(i));
        }
        return ret;
    }

    private Geometry BuildGeometry(XmlNode xmlNode)
    {
        foreach (XmlNode i in xmlNode.ChildNodes)
        {
            if (i.Name.Equals("gml:MultiSurface", StringComparison.OrdinalIgnoreCase))
                return BuildMultiSurface(i);
            if (i.Name.Equals("gml:MultiLineString", StringComparison.OrdinalIgnoreCase))
                return BuildMultiLineString(i);
            if (i.Name.Equals("gml:Point", StringComparison.OrdinalIgnoreCase))
                return BuildPoint(i);
        }
        throw new NotImplementedException();
    }

    private Geometry BuildPoint(XmlNode xmlNode)
    {
        string[] ss = xmlNode.SelectSingleNode("gml:pos",nsmgr).InnerText.Split(' ');
        return new Point(double.Parse(ss[1]), double.Parse(ss[0]));
    }

    private Geometry BuildMultiLineString(XmlNode xmlNode)
    {
        MultiLineString ret = new MultiLineString();
        foreach (XmlNode i in xmlNode.SelectNodes("gml:lineStringMember/gml:LineString", nsmgr))
        {
            ret.LineStrings.Add(BuildLineString(i));
        }
        return ret;
    }

    private LineString BuildLineString(XmlNode xmlNode)
    {
        return new LineString(BuildPointList(xmlNode.SelectSingleNode("gml:posList", nsmgr)));
    }

    private Geometry BuildMultiSurface(XmlNode xmlNode)
    {
        MultiPolygon ret = new MultiPolygon();
        foreach (XmlNode i in xmlNode.SelectNodes("gml:surfaceMember/gml:Polygon", nsmgr))
        {
            ret.Polygons.Add(BuildPolygon(i));
        }
        return ret;
    }

    //这里没有处理内环interior
    private Polygon BuildPolygon(XmlNode xmlNode)
    {
        Polygon ret = new Polygon();
        XmlNode n = xmlNode.SelectSingleNode("gml:exterior/gml:LinearRing", nsmgr);
        if (n == null)
            throw new Exception();
        ret.ExteriorRing = BuildLinearRing(n);
        return ret;
    }

    private LinearRing BuildLinearRing(XmlNode xmlNode)
    {
        return new LinearRing(BuildPointList(xmlNode.SelectSingleNode("gml:posList", nsmgr)));
    }

    private List<Point> BuildPointList(XmlNode xmlNode)
    {
        if (xmlNode == null)
            throw new Exception();
        List<Point> ret = new List<Point>();
        string[] ss = xmlNode.InnerText.Split(' ');
        for (int i = 1; i < ss.Length; )
        {
            ret.Add(new Point(double.Parse(ss[i]), double.Parse(ss[i - 1])));
            i += 2;
        }
        return ret;
    }
}